using UnityEngine;
using UnityEngine.UI;
using UnityEngine.XR;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;
using MeshVR;
using MVR;
using MVR.FileManagementSecure;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;

// StudioMaestro v1.0
//
// Handles person position, pose, window camera and light presets to quickly shot a character

namespace StudioMaestro
{
    public class StudioMaestroPlugin : MVRScript
    {	
		// Static string
		private static string PLUGIN_PATH;
		private static string PACKAGE_PATH;
		private static string SCENE_PACKAGE_PATH;
		private static string DEFAULT_PRESET_LABEL = "Preset name...";
		private static string EXCEPTION_PREFIX = PLUGIN_NAME + " - Exception caught: ";
		private static string PRESETS_LIGHTS_PATH = "Saves\\PluginData\\StudioMaestro\\Lights_Presets";
		private static string PRESETS_LIGHTSLOCAL_PATH = "Saves\\PluginData\\StudioMaestro\\LightsLocal_Presets";
		private static string PRESETS_GLOBALLIGHT_PATH = "Saves\\PluginData\\StudioMaestro\\GlobalLight_Presets";
		private static string PRESETS_WINCAM_PATH = "Saves\\PluginData\\StudioMaestro\\WinCam_Presets";
		private static string PRESETS_POSE_PATH = "Saves\\PluginData\\StudioMaestro\\Pose_Presets";
		private static string PRESETS_SETS_PATH = "Custom\\SubScene\\StudioMaestro\\STMA_Set";
		private static string BACKUP_PATH = "Saves\\PluginData\\StudioMaestro\\_Backups";
		private static string DEFAULTS_PATH = "Saves\\PluginData\\StudioMaestro\\_Defaults";
		
		private static string DEFAULT_SS_EMPTY_PATH = "Saves\\PluginData\\StudioMaestro\\_Defaults\\default-supershot-empty-preset.vap";
		

		private static string PRESET_DEFAULT_INACTIVE_POSE;
		
		// Misc static strings for atoms and so on
		private static string PLUGIN_NAME = "StudioMaestro";
		private static string LIGHTS_PARENT_NAME = "STMA_Lights_Parent";
		private static string LIGHTS_PREFIX = "STMA_Light_";
		private static string CHARS_PREFIX = "STMA_Char_";

		// Errors alerts
		private static string TXT_ERROR_ESSENTIALS = "<b><color=yellow>STUDIO MAESTRO\nMACGRUBER'S ESSENTIAL ERROR</color></b>\n"+
		"<size=32>You need to download MacGruber's Essentials 16 to be able to use the Window Camera screenshot feature.\n\n"+
		"Not having MacGruber's essentials might trigger warnings or errors on load but are not critical and StudioMaestro can be used normally.\n\n</size>";
		
		private static string TXT_ERROR_SUPERSHOT = "<b><color=yellow>STUDIO MAESTRO SUPERSHOT ERROR</color></b>\n"+
		"<size=32>The StudioMaestro SuperShot atom has been found, but the plugin couldn't be accessed.\n\n"+
		"The reason might be because you have modified plugins order OR You have removed the plugin and saved the scene.\n\n" +
		"Click Ok to restore the default SuperShot plugin, or click cancel (if you cancel, the screenshot button will not work).\n\n</size>";

		// Help sections
		private static string TXT_HELP_STUDIOMAESTRO = "<b><color=green>STUDIO MAESTRO HELP</color></b>\n"+
		"<size=32>Studio Maestro is a session plugin dedicated to\nVirtual Photography made and thought for Juno's workflow.\n\n"+
		"The concept revolves around the Window Camera position\n+ lights, pose and set presets.\n\n" +
		"Use the tiny help buttons for some guidance on how to use the plugin. Read the documentation on the hub's release page for advanced tips and recommandations about the plugin\n\n</size>";

		private static string TXT_HELP_LIGHTPRESET = "<b><color=green>LIGHT PRESET HELP</color></b>\n"+
		"<size=32>This is the light preset section, you can load lighting preset you already have saved by selecting an item in the dropdown below this help button.\n\n"+
		"The search button allows you to browse through existing presets and search by keywords if you have a lot of light presets.\n\n" +
		"If you save or load a light preset, you will be able to rotate it with the slider in the right column. You can find more infos in the rotate light rig help.\n\n" +
		"Use SAVE and DELETE to manage your light presets.</size>";
		
		private static string TXT_HELP_WINCAMPRESET = "<b><color=green>WINDOW CAMERA PRESET HELP</color></b>\n"+
		"<size=32>This is the window camera preset section, you can load Window Camera positions you already have saved by selecting an item in the dropdown below this help button.\n\n"+
		"Use SAVE and DELETE to manage your camera positions.</size>";

		private static string TXT_HELP_POSEPRESET = "<b><color=green>POSE PRESET HELP</color></b>\n"+
		"<size=32>This is the pose preset section, you can load pose presets you already have saved by selecting an item in the dropdown below this help button.\n\n"+
		"The search button allows you to browse through existing presets and search by keywords if you have a lot of poses.\n\n" +
		"Use SAVE and DELETE to manage your poses.\n\n" +
		"The up and down blue arrows are made to create a quick snapshot of a pose. The Up arrow will restore the last snapshot. " +
		"Down arrow will save a snapshot. This snapshot is also always written everytime you load a pose preset to work as a safety net.\n\n" +
		"Pose presets handle multiple characters. If you don't have the correct character count in your scene, the pose system will restore as much as it can. " +
		"The pose system will not create characters for you, you have to do that by yourself.</size>";

		private static string TXT_HELP_SETPRESET = "<b><color=green>SET PRESET HELP</color></b>\n"+
		"<size=30>This is the set preset section, you can load set presets you already have saved by selecting an item in the dropdown below this help button.\n\n"+
		"The search button allows you to browse through existing presets and search by keywords if you have a lot of sets.\n\n" +
		"Sets are SubScenes saved in \\Custom\\SubScenes\\StudioMaestro\\STMA_Set.\n\n" +
		"Use CLEAR to remove the set from the scene. Use REFRESH LIST to update the list if you have saved new SubScenes.\n\n" +
		"RECOMMANDATIONS:\n\n" +
		"Create your SubScenes beforehand, save them with the proper settings ( StudioMaestro/STMA_SET/MySetXX for Creator Name, Subscene Signature and Save name ), then " +
		"use the Load Set Preset after you have refresh the list using the REFRESH LIST button.</size>";

		private static string TXT_HELP_SYNCPRESETS = "<b><color=green>SYNCHRONIZED PRESET LOADING HELP</color></b>\n"+
		"<size=32>Synchronizing presets loading will load presets with the same name in any other category.\n\n"+
		"If you load any of the categories in the left column and have one of these checkboxes enabled, "+
		"the other categories will also be loaded if they have a preset of the same name already saved.</size>";

		private static string TXT_HELP_ROTATELIGHTRIG = "<b><color=green>LIGHT RIG ROTATION HELP</color></b>\n"+
		"<size=32>This slider allows you to rotate the light rig.\n\n Rotating the light rig will only work if you have loaded or saved a light preset. "+
		"Or loaded a scene already created with Studio Maestro and a proper light rig.\n\n" + 
		"If the light rig does not rotate,\nsimply save or load an existing preset.\n\n" +
		"The light rig revolves around the origin of the scene (0,0,0). So keep that in mind when when rotating over the X and Z axis.</size>";
		
		private static string TXT_HELP_CAMTWEAKS = "<b><color=green>CAMERA AND EYE TARGET TWEAKS HELP</color></b>\n"+
		"<size=32>The camera tweaks allow you to quickly update\nsome window camera settings.\n\n"+
		"You can update the FOV and move or rotate the camera through the 6DoF controls.\n\n" + 
		"The left 6DOF controls are for the position.\nThe right 6DOF controls are for the rotation.\n\n" +
		"The Tweak value controls how big the position or rotation tweak is. A lower value will move or rotate the window camera less than a higher value.\n\n" +
		"The Eye Target tweaks are working the exact same way,\nyou just only have the position.\n\nYou can also use the orange buttons to set the look at mode to target, move the target to the camera or your current position directly.</size>";

		private static string TXT_WARNING_NOEYETARGETSELECTION = "<b><color=yellow>STUDIO MAESTRO EYE TARGET ERROR</color></b>\nSelect a character in the dropdown above the tweak buttons before trying to move or change Eye Target settings.";

		// String choosers lists
		protected List<string> lightsPresetsChoices = new List<String>();
		protected List<string> wincamPresetsChoices = new List<String>();
		protected List<string> posePresetsChoices = new List<String>();
		protected List<string> setPresetsChoices = new List<String>();
		protected List<string> eyeTargetCharChoices = new List<String>();
		
		// UI lists
		protected List<UIDynamic> ignoreLayoutUI = new List<UIDynamic>();
		protected List<UIDynamic> tabCamTweaksUIs = new List<UIDynamic>();
		protected List<UIDynamic> tabEyeTargetTweaksUIs = new List<UIDynamic>();
		
		// UI Buttons (specific)
		protected UIDynamicButton uiBtnCamTweaks;
		protected UIDynamicButton uiBtnEyeTargetTweaks;
		
		// Ignores for the JSON saves
		protected List<string> ignoreStorablesLights = new List<String>() { "animation", "AtomControl", "CollisionTrigger", "Light", "materials", "PhysicsMaterialControl", "PluginManager", "Preset", "scale" };
		protected List<string> ignoreStorablesLightLocal = new List<String>() { "animation", "AtomControl", "CollisionTrigger", "control", "materials", "PhysicsMaterialControl", "Preset", "scale" };
		protected List<string> ignoreStorablesWinCam = new List<String>() { "animation", "AtomControl", "CollisionTrigger", "DisplayAnimation", "DisplayControl", "DisplayPlanesScale", "materials", "PhysicsMaterialControl", "PluginManager", "Preset", "scale" };
		
		// Atoms
		protected Atom WinCamAtom;
		protected CameraControl WinCamControl;
		protected Atom LightRig;
		
		// Miscs JSONStorables
		protected FreeControllerV3 charEyeTarget;
		
		// Transforms
		protected Transform WinCamPreviewBackground;
		protected Transform WinCamPreviewImage;
		
		// Some vectors
		protected Vector3 WinCamPreviewImgOriginalScale = new Vector3(0.02222f, 0.0125f, 0.0125f);
		protected Vector3 WinCamPreviewImgOriginalPos = new Vector3(-0.2f, 0.23f, 0.0f);
		protected Vector3 WinCamPreviewBgOriginalScale = new Vector3(0.02322f, 0.0135f, 0.0135f);
		protected Vector3 WinCamPreviewBgOriginalPos = new Vector3(-0.2f, 0.23f, -0.002f);
		protected Vector3 WinCamPreviewOriginalRot = new Vector3(90f,0,0);
		
		// Storables
		public JSONStorableString PresetName;
		protected InputField PresetNameInputField;
		public JSONStorableStringChooser LightsPresetChooser;
		public JSONStorableStringChooser WinCamPresetChooser;
		public JSONStorableStringChooser PosePresetChooser;
		public JSONStorableStringChooser SetPresetChooser;
		
		public JSONStorableBool SyncLightPresetLoading;
		public JSONStorableBool SyncWinCamPresetLoading;
		public JSONStorableBool SyncPosePresetLoading;
		public JSONStorableBool SyncSetPresetLoading;

		public JSONStorableFloat LightRigRotationX;
		public JSONStorableFloat LightRigRotationY;
		public JSONStorableFloat LightRigRotationZ;
		
		public JSONStorableFloat WinCamFOV;
		public JSONStorableFloat WinCamTweakVal;
		public JSONStorableFloat WinCamExposure;
		
		public JSONStorableFloat EyeTargetTweakVal;
		public JSONStorableStringChooser EyeTargetCharacter;
		
		// Overflowing buttons
		private UIDynamicButton BtnTogglePreview;
		private UIDynamicButton BtnFlipRatio;
		private UIDynamicButton BtnToggleCam;
		private UIDynamicButton BtnSaveScreenshot;
		
		// JSON Classes
		private JSONClass defaultInactivePoseJSON;
		private JSONClass defaultBrowserSettings;
		
		// Fake triggers (ima gonna use that to trigger "forbidden" actions)
		public EventTrigger Load_FakeTrigger;
		private List<EventTrigger> allTriggers;
		
		// Some colors
		protected Color tabTweaksBgColor = new Color(0.5f, 0.6f, 0.7f, 1f);
		protected Color tabTweaksSelBgColor = new Color(0.1f, 0.4f, 0.7f, 1f);

		// Some flags
		protected bool previewIsPortrait = true;
		
		public override void Init()
        {			 
            try
            {
	            if (containingAtom.type != "SessionPluginManager")
	            {
		            SuperController.singleton.Alert("<b><color=green>"+PLUGIN_NAME+"</color></b>\nStudio Maestro is a session plugin.\nPlease add it in your session plugins.", () => {});
		            return;
	            }
	            
	            PLUGIN_PATH = Utils.GetPluginPath(this);
	            PRESET_DEFAULT_INACTIVE_POSE = PLUGIN_PATH + "\\defaults\\default-inactive-pose.json";
	            
	            defaultInactivePoseJSON = SuperController.singleton.LoadJSON(PRESET_DEFAULT_INACTIVE_POSE) as JSONClass;
	            
	            // Temporary vars
	            HSVColor hsvcDefault = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
	            HSVColor hsvcDefaultOverClick = HSVColorPicker.RGBToHSV(0.55f, 0.55f, 0.55f);
	            HSVColor hsvcTxtDefault = HSVColorPicker.RGBToHSV(0.2f, 0.2f, 0.2f);
	            UIDynamicTextField tmpTextfield;
	            UIDynamicPopup tmpPopup;
	            UIDynamicSlider tmpSlider;
	            UIDynamicColorPicker tmpColor;
	            UIDynamicButton tmpButton;
	            UIDynamicToggle tmpToggle;
	            UIDynamic tmpSpacer;
	            
	            Utils.CreateStaticDescriptionText(this, "STUDIO MAESTRO Title","<color=#000><size=35><b>STUDIO MAESTRO</b></size></color>",false,40,TextAnchor.MiddleLeft);
	            
	            tmpButton = CreateButton("HELP",false);
	            tmpButton.button.onClick.AddListener( () => { SuperController.singleton.Alert(TXT_HELP_STUDIOMAESTRO, () => {}); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.7f, 0.9f, 0f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(0.1f, 0.1f, 0.1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 20);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(80, 30), new Vector2(340, -28));

	            tmpSpacer = CreateSpacer(false);
	            Utils.SetupSpacer(tmpSpacer, 25f);
	            
	            PresetName = new JSONStorableString("Global_Opt_TxtHeaderText", "_default_") {isStorable=false,isRestorable=false};
	            tmpTextfield = CreateTextField(PresetName,false);
	            Utils.SetupTextField(tmpTextfield, 80f, false, false,40);
	            PresetNameInputField = tmpTextfield.UItext.gameObject.AddComponent<InputField>();
	            PresetNameInputField.textComponent = tmpTextfield.UItext;
	            PresetNameInputField.lineType = InputField.LineType.SingleLine;
	            PresetName.valNoCallback = DEFAULT_PRESET_LABEL;
	            PresetNameInputField.text = DEFAULT_PRESET_LABEL;				
	            PresetNameInputField.onValueChanged.AddListener( delegate { OnPresetNameChanged(); } );
	            PresetNameInputField.onEndEdit.AddListener(delegate { OnPresetNameChanged(); } );
	            
	            tmpSpacer = CreateSpacer(false);
	            Utils.SetupSpacer(tmpSpacer, 25f);    
	            
	            LightsPresetChooser = new JSONStorableStringChooser("LightsPresetSelector", lightsPresetsChoices, "", "Load Light Preset") {isStorable=false,isRestorable=false};
	            LightsPresetChooser.setCallbackFunction += (val) => { OnSelectLightsPreset(val); };
	            RegisterStringChooser(LightsPresetChooser);
	            tmpPopup = CreateScrollablePopup(LightsPresetChooser, false);
	            
	            tmpButton = CreateButton("HELP",false);
	            tmpButton.button.onClick.AddListener( () => { SuperController.singleton.Alert(TXT_HELP_LIGHTPRESET, () => {}); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.7f, 0.9f, 0f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(0.1f, 0.1f, 0.1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 20);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(80, 30), new Vector2(12, -226));

	            tmpButton = CreateButton("SAVE",false);
	            tmpButton.button.onClick.AddListener( () => { OnSaveLightsPreset(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.1f, 0.7f, 0.4f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 24);
	            Utils.SetupButtonWithLayout(tmpButton, new Vector2(80, 40));
	            
	            tmpButton = CreateButton("DELETE",false);
	            tmpButton.button.onClick.AddListener( () => { OnDeleteLightsPreset(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.7f, 0.3f, 0.3f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 24);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(120, 40), new Vector2(106, -375));
	            
	            tmpButton = CreateButton("SEARCH",false);
	            tmpButton.button.onClick.AddListener( () => { OnSearchLight(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 24);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(120, 40), new Vector2(410, -225));
	            
	            tmpSpacer = CreateSpacer(false);
	            Utils.SetupSpacer(tmpSpacer, 60f);
	            
	            WinCamPresetChooser = new JSONStorableStringChooser("WinCamPresetSelector", lightsPresetsChoices, "", "Load Cam Preset") {isStorable=false,isRestorable=false};
	            WinCamPresetChooser.setCallbackFunction += (val) => { OnSelectWinCamPreset(val); };
	            RegisterStringChooser(WinCamPresetChooser);
	            tmpPopup = CreateScrollablePopup(WinCamPresetChooser, false);
	            
	            tmpButton = CreateButton("HELP",false);
	            tmpButton.button.onClick.AddListener( () => { SuperController.singleton.Alert(TXT_HELP_WINCAMPRESET, () => {}); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.7f, 0.9f, 0f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(0.1f, 0.1f, 0.1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 20);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(80, 30), new Vector2(12, -472));

	            tmpButton = CreateButton("SAVE",false);
	            tmpButton.button.onClick.AddListener( () => { OnSaveWinCamPreset(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.1f, 0.7f, 0.4f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 24);
	            Utils.SetupButtonWithLayout(tmpButton, new Vector2(80, 40));
	            
	            tmpButton = CreateButton("DELETE",false);
	            tmpButton.button.onClick.AddListener( () => { OnDeleteWinCamPreset(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.7f, 0.3f, 0.3f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 24);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(120, 40), new Vector2(106, -620));
	            
	            tmpSpacer = CreateSpacer(false);
	            Utils.SetupSpacer(tmpSpacer, 60f);
	            
	            PosePresetChooser = new JSONStorableStringChooser("PosePresetSelector", posePresetsChoices, "", "Load Pose Preset") {isStorable=false,isRestorable=false};
	            PosePresetChooser.setCallbackFunction += (val) => { OnSelectPosePreset(val); };
	            RegisterStringChooser(PosePresetChooser);
	            tmpPopup = CreateScrollablePopup(PosePresetChooser, false);
	            
	            tmpButton = CreateButton("HELP",false);
	            tmpButton.button.onClick.AddListener( () => { SuperController.singleton.Alert(TXT_HELP_POSEPRESET, () => {}); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.7f, 0.9f, 0f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(0.1f, 0.1f, 0.1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 20);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(80, 30), new Vector2(12, -716));

	            tmpButton = CreateButton("SEARCH",false);
	            tmpButton.button.onClick.AddListener( () => { OnSearchPose(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 24);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(120, 40), new Vector2(410, -715));
	            
	            tmpButton = CreateButton("SAVE",false);
	            tmpButton.button.onClick.AddListener( () => { OnSavePosePreset(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.1f, 0.7f, 0.4f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 24);
	            Utils.SetupButtonWithLayout(tmpButton, new Vector2(80, 40));
	            
	            tmpButton = CreateButton("DELETE",false);
	            tmpButton.button.onClick.AddListener( () => { OnDeletePosePreset(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.7f, 0.3f, 0.3f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 24);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(120, 40), new Vector2(106, -865));
	            
	            tmpButton = CreateButton("▲",false);
	            tmpButton.button.onClick.AddListener( () => { OnRestoreLastPoseSnapshot(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(440, -865));
	            
	            tmpButton = CreateButton("▼",false);
	            tmpButton.button.onClick.AddListener( () => { OnSnapshotCurrentPose(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(490, -865));

	            tmpSpacer = CreateSpacer(false);
	            Utils.SetupSpacer(tmpSpacer, 60f);
	            
	            SetPresetChooser = new JSONStorableStringChooser("SetsPresetSelector", setPresetsChoices, "", "Load Set Preset") {isStorable=false,isRestorable=false};
	            SetPresetChooser.setCallbackFunction += (val) => { OnSelectSetPreset(val); };
	            RegisterStringChooser(SetPresetChooser);
	            tmpPopup = CreateScrollablePopup(SetPresetChooser, false);

	            tmpButton = CreateButton("HELP",false);
	            tmpButton.button.onClick.AddListener( () => { SuperController.singleton.Alert(TXT_HELP_SETPRESET, () => {}); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.7f, 0.9f, 0f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(0.1f, 0.1f, 0.1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 20);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(80, 30), new Vector2(12, -960));
	            
	            tmpButton = CreateButton("SEARCH",false);
	            tmpButton.button.onClick.AddListener( () => { OnSearchSet(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 24);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(120, 40), new Vector2(410, -960));

	            tmpButton = CreateButton("CLEAR",false);
	            tmpButton.button.onClick.AddListener( () => { OnClearSet(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 24);
	            Utils.SetupButtonWithLayout(tmpButton, new Vector2(90, 40));
	            
	            tmpButton = CreateButton("REFRESH LIST",false);
	            tmpButton.button.onClick.AddListener( () => { OnRefreshSetList(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 24);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(190, 40), new Vector2(115, -1110));

	            SyncLightPresetLoading = new JSONStorableBool("SyncLightPresetLoading", false){isStorable=false,isRestorable=false};
	            tmpToggle = CreateToggle(SyncLightPresetLoading,true);
	            tmpToggle.label = "Sync Light preset loading";
	            
	            SyncWinCamPresetLoading = new JSONStorableBool("SyncWinCamPresetLoading", false){isStorable=false,isRestorable=false};
	            tmpToggle = CreateToggle(SyncWinCamPresetLoading,true);
	            tmpToggle.label = "Sync Win Cam preset loading";
	            
	            SyncPosePresetLoading = new JSONStorableBool("SyncPosePresetLoading", false){isStorable=false,isRestorable=false};
	            tmpToggle = CreateToggle(SyncPosePresetLoading,true);
	            tmpToggle.label = "Sync Pose preset loading";
	            
	            SyncSetPresetLoading = new JSONStorableBool("SyncSetPresetLoading", false){isStorable=false,isRestorable=false};
	            tmpToggle = CreateToggle(SyncSetPresetLoading,true);
	            tmpToggle.label = "Sync Set preset loading";

	            tmpButton = CreateButton("HELP",true);
	            tmpButton.button.onClick.AddListener( () => { SuperController.singleton.Alert(TXT_HELP_SYNCPRESETS, () => {}); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.7f, 0.9f, 0f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(0.1f, 0.1f, 0.1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 20);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(80, 30), new Vector2(440, -35));
	            
	            LightRigRotationX = new JSONStorableFloat("RotateLightRigX", 0f, 0f, 360f){isStorable=false,isRestorable=false};
	            LightRigRotationX.setCallbackFunction += (val) => { OnRotateLightRig(); };
	            tmpSlider = CreateSlider(LightRigRotationX, true);
	            tmpSlider.valueFormat = "F2";
	            tmpSlider.label = "Rotate Light Rig (X)";
	            
	            LightRigRotationY = new JSONStorableFloat("RotateLightRigY", 0f, 0f, 360f){isStorable=false,isRestorable=false};
	            LightRigRotationY.setCallbackFunction += (val) => { OnRotateLightRig(); };
	            tmpSlider = CreateSlider(LightRigRotationY, true);
	            tmpSlider.valueFormat = "F2";
	            tmpSlider.label = "Rotate Light Rig (Y)";
	            
	            LightRigRotationZ = new JSONStorableFloat("RotateLightRigZ", 0f, 0f, 360f){isStorable=false,isRestorable=false};
	            LightRigRotationZ.setCallbackFunction += (val) => { OnRotateLightRig(); };
	            tmpSlider = CreateSlider(LightRigRotationZ, true);
	            tmpSlider.valueFormat = "F2";
	            tmpSlider.label = "Rotate Light Rig (Z)";
	            
	            tmpButton = CreateButton("HELP",true);
	            tmpButton.button.onClick.AddListener( () => { SuperController.singleton.Alert(TXT_HELP_ROTATELIGHTRIG, () => {}); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.7f, 0.9f, 0f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(0.1f, 0.1f, 0.1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 20);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(80, 30), new Vector2(280, -297));

	            // Utils.CreateStaticDescriptionText(this, "CAMERA TWEAKS TITLE","<color=#000><size=30><b>Camera Tweaks</b></size></color>",true,30,TextAnchor.MiddleLeft);
	            
	            uiBtnCamTweaks = CreateButton("CAMERA",true);
	            uiBtnCamTweaks.button.onClick.AddListener( () => { OnShowTabCamTweaks(1); } );
	            Utils.SetButtonColor(uiBtnCamTweaks, tabTweaksSelBgColor);
	            Utils.SetButtonTextColor(uiBtnCamTweaks, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(uiBtnCamTweaks, 22);
	            Utils.SetupButtonWithLayout(uiBtnCamTweaks, new Vector2(140, 30));
	            
	            uiBtnEyeTargetTweaks = CreateButton("EYE TARGET",true);
	            uiBtnEyeTargetTweaks.button.onClick.AddListener( () => { OnShowTabCamTweaks(2); } );
	            Utils.SetButtonColor(uiBtnEyeTargetTweaks, tabTweaksBgColor);
	            Utils.SetButtonTextColor(uiBtnEyeTargetTweaks, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(uiBtnEyeTargetTweaks, 22);
	            Utils.SetupButtonWithoutLayout(uiBtnEyeTargetTweaks, new Vector2(180, 30), new Vector2(160, -690));
	            
	            tmpButton = CreateButton("HELP",true);
	            tmpButton.button.onClick.AddListener( () => { SuperController.singleton.Alert(TXT_HELP_CAMTWEAKS, () => {}); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.7f, 0.9f, 0f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(0.1f, 0.1f, 0.1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 20);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(80, 30), new Vector2(440, -690));
	            
	            WinCamFOV = new JSONStorableFloat("WinCamFOV", 40f, 10f, 100f){isStorable=false,isRestorable=false};
	            WinCamFOV.setCallbackFunction += (val) => { OnChangeWinCamFOV(val); };
	            tmpSlider = CreateSlider(WinCamFOV, true);
	            tmpSlider.valueFormat = "F1";
	            tmpSlider.label = "Camera FOV";
	            tabCamTweaksUIs.Add((UIDynamic) tmpSlider);
	            
	            WinCamTweakVal = new JSONStorableFloat("WinCamTweakVal", 0.010f, 0.001f, 0.250f){isStorable=false,isRestorable=false};
	            tmpSlider = CreateSlider(WinCamTweakVal, true);
	            tmpSlider.valueFormat = "F3";
	            tmpSlider.label = "Tweak value";
	            tabCamTweaksUIs.Add((UIDynamic) tmpSlider);
	            
	            // Camera tweaks buttons
	            tmpSpacer = CreateSpacer(true);
	            Utils.SetupSpacer(tmpSpacer, 30f);
	            tabCamTweaksUIs.Add((UIDynamic) tmpSpacer);
	            
	            // Camera location
	            tmpButton = CreateButton("◄",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakCamLeft(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithLayout(tmpButton, new Vector2(40, 40));
	            tabCamTweaksUIs.Add((UIDynamic) tmpButton);
	            
	            tmpButton = CreateButton("▲",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakCamUp(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(55, -1010));
	            tabCamTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
	            tmpButton = CreateButton("▼",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakCamDown(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(55, -1055));
	            tabCamTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);

	            tmpButton = CreateButton("►",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakCamRight(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(100, -1055));
	            tabCamTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
	            tmpButton = CreateButton("↑",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakCamForward(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(145, -1010));
	            tabCamTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
	            tmpButton = CreateButton("↓",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakCamBackward(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(145, -1055));
	            tabCamTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
	            // Camera rotation
	            tmpButton = CreateButton("↶",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakCamRotZNeg(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(300, -1055));
	            tabCamTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
	            tmpButton = CreateButton("↑",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakCamRotXPos(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(345, -1010));
	            tabCamTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
	            tmpButton = CreateButton("↓",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakCamRotXNeg(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(345, -1055));
	            tabCamTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);

	            tmpButton = CreateButton("↷",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakCamRotZPos(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(390, -1055));
	            tabCamTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
	            tmpButton = CreateButton("↻",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakCamRotYNeg(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(435, -1010));
	            tabCamTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
	            tmpButton = CreateButton("↺",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakCamRotYPos(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(435, -1055));
	            tabCamTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
	            WinCamExposure = new JSONStorableFloat("WinCamExposure", 1f, 0f, 10f){isStorable=false,isRestorable=false};
	            tmpSlider = CreateSlider(WinCamExposure, true);
	            WinCamExposure.setCallbackFunction += (val) => { OnChangeWinCamExposure(val); };
	            tmpSlider.valueFormat = "F2";
	            tmpSlider.label = "Camera Exposure";
	            tabCamTweaksUIs.Add((UIDynamic) tmpSlider);
	            
				// Eye Target tweaks
				EyeTargetCharacter = new JSONStorableStringChooser("EyeTargetCharacter", eyeTargetCharChoices, "", "Character") {isStorable=false,isRestorable=false};
				EyeTargetCharacter.setCallbackFunction += (val) => { OnChangeEyeTargetChar(val); };
				RegisterStringChooser(EyeTargetCharacter);
				tmpPopup = CreateScrollablePopup(EyeTargetCharacter, true);
				tabEyeTargetTweaksUIs.Add((UIDynamic) tmpPopup);
				
				EyeTargetTweakVal = new JSONStorableFloat("WinCamTweakVal", 0.010f, 0.001f, 0.250f){isStorable=false,isRestorable=false};
				tmpSlider = CreateSlider(EyeTargetTweakVal, true);
				tmpSlider.valueFormat = "F3";
				tmpSlider.label = "Tweak value";
				tabEyeTargetTweaksUIs.Add((UIDynamic) tmpSlider);

				// Eye target buttons
				tmpSpacer = CreateSpacer(true);
				Utils.SetupSpacer(tmpSpacer, 50f);
				tabEyeTargetTweaksUIs.Add((UIDynamic) tmpSpacer);
				
				// Eye target location
	            tmpButton = CreateButton("◄",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakETLeft(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithLayout(tmpButton, new Vector2(40, 40));
	            tabEyeTargetTweaksUIs.Add((UIDynamic) tmpButton);
	            
	            tmpButton = CreateButton("▲",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakETUp(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(55, -1010));
	            tabEyeTargetTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
	            tmpButton = CreateButton("▼",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakETDown(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(55, -1055));
	            tabEyeTargetTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);

	            tmpButton = CreateButton("►",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakETRight(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(100, -1055));
	            tabEyeTargetTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
	            tmpButton = CreateButton("↑",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakETForward(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(145, -1010));
	            tabEyeTargetTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
	            tmpButton = CreateButton("↓",true);
	            tmpButton.button.onClick.AddListener( () => { OnTweakETBackward(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.2f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 34);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(40, 40), new Vector2(145, -1055));
	            tabEyeTargetTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);

	            // Eye Target modes
	            tmpButton = CreateButton("SET LOOK AT EYE TARGET",true);
	            tmpButton.button.onClick.AddListener( () => { OnSetEyeTargetLookAt(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.8f, 0.5f, 0.1f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 22);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(310, 40), new Vector2(210, -1005));
	            tabEyeTargetTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
	            tmpButton = CreateButton("MOVE TO WIN CAMERA",true);
	            tmpButton.button.onClick.AddListener( () => { OnMoveEyeTargetToWinCamera(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.8f, 0.5f, 0.1f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 22);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(310, 40), new Vector2(210, -1050));
	            tabEyeTargetTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
	            tmpButton = CreateButton("MOVE TO MY POSITION",true);
	            tmpButton.button.onClick.AddListener( () => { OnMoveEyeTargetToMyPosition(); } );
	            Utils.SetButtonColor(tmpButton, new Color(0.8f, 0.5f, 0.1f, 1f));
	            Utils.SetButtonTextColor(tmpButton, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(tmpButton, 22);
	            Utils.SetupButtonWithoutLayout(tmpButton, new Vector2(310, 40), new Vector2(210, -1095));
	            tabEyeTargetTweaksUIs.Add((UIDynamic) tmpButton); ignoreLayoutUI.Add((UIDynamic) tmpButton);
	            
				// Showing the first tab for the Camera tweaks area
				OnShowTabCamTweaks(1);
		            
	            /** OVERFLOWING BUTTONS **/
	            BtnTogglePreview = CreateButton("TOGGLE\nPREVIEW",true);
	            BtnTogglePreview.button.onClick.AddListener( () => { OnTogglePreview(); } );
	            Utils.SetButtonColor(BtnTogglePreview, new Color(0.1f, 0.4f, 0.7f, 1f));
	            Utils.SetButtonTextColor(BtnTogglePreview, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(BtnTogglePreview, 28);
	            
	            BtnFlipRatio = CreateButton("FLIP PREVIEW\nORIENTATION",true);
	            BtnFlipRatio.button.onClick.AddListener( () => { OnFlipPreviewAspectRatio(); } );
	            Utils.SetButtonColor(BtnFlipRatio, new Color(0.4f, 0.4f, 0.4f, 1f));
	            Utils.SetButtonTextColor(BtnFlipRatio, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(BtnFlipRatio, 28);
	            
	            BtnToggleCam = CreateButton("TOGGLE\nCAMERA",true);
	            BtnToggleCam.button.onClick.AddListener( () => { OnToggleWinCam(); } );
	            Utils.SetButtonColor(BtnToggleCam, new Color(0.6f, 0.4f, 0.4f, 1f));
	            Utils.SetButtonTextColor(BtnToggleCam, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(BtnToggleCam, 28);
	            
	            BtnSaveScreenshot = CreateButton("SAVE\nSCREENSHOT",true);
	            BtnSaveScreenshot.button.onClick.AddListener( () => { OnSaveSuperShotScreenshot(); } );
	            Utils.SetButtonColor(BtnSaveScreenshot, new Color(0.1f, 0.6f, 0.6f, 1f));
	            Utils.SetButtonTextColor(BtnSaveScreenshot, new Color(1f, 1f, 1f, 1f));
	            Utils.SetButtonTextSize(BtnSaveScreenshot, 28);

	            // Maestro Presets folder
	            FileManagerSecure.CreateDirectory(PRESETS_LIGHTS_PATH);
	            FileManagerSecure.CreateDirectory(PRESETS_LIGHTSLOCAL_PATH);
	            FileManagerSecure.CreateDirectory(PRESETS_GLOBALLIGHT_PATH);
	            FileManagerSecure.CreateDirectory(PRESETS_WINCAM_PATH);
	            FileManagerSecure.CreateDirectory(PRESETS_POSE_PATH);
	            FileManagerSecure.CreateDirectory(PRESETS_SETS_PATH);
	            FileManagerSecure.CreateDirectory(BACKUP_PATH);
	            FileManagerSecure.CreateDirectory(DEFAULTS_PATH);
	            
	            // Creating the default SuperShot Preset
	            if (!FileManagerSecure.FileExists(DEFAULT_SS_EMPTY_PATH))
	            {
		            JSONClass emptyDefaultPreset = SuperController.singleton.LoadJSON(PLUGIN_PATH + "/defaults/default-supershot-empty-preset.vap") as JSONClass;
		            SuperController.singleton.SaveJSON(emptyDefaultPreset, DEFAULT_SS_EMPTY_PATH);
	            }
	            
	            // Fake triggers
	            Load_FakeTrigger = new EventTrigger(this, "Load_FakeTrigger");
	            
	            // Loading triggers assets
	            SimpleTriggerHandler.LoadAssets();
	            
	            // Handlers
	            SuperController.singleton.onSceneLoadedHandlers += OnSceneLoaded;
	            
	            // Saving browser defaults
	            // I don't know why, some browsers in VAM do not set their value properly back to what is needed
	            // so you end not seeing your files and saves correctly
	            // hence saving the default and restoring them everytime the browser is opened
	            SaveBrowserDefaults();

            }
            catch(Exception e)
            {
                SuperController.LogError(EXCEPTION_PREFIX + e);
            }
		}
		
		void Start()
        {
			try
			{
				Utils.SetupButtonOverflowing(BtnTogglePreview, new Vector2(220, 100), new Vector2(0, -750));
				Utils.SetupButtonOverflowing(BtnFlipRatio, new Vector2(220, 100), new Vector2(0, -850));
				Utils.SetupButtonOverflowing(BtnToggleCam, new Vector2(220, 100), new Vector2(0, -950));
				Utils.SetupButtonOverflowing(BtnSaveScreenshot, new Vector2(220, 100), new Vector2(0, -1100));

				// Grabbing my components
				// This happens directly here or through the scene load handler
				OnSceneLoaded();
			}
            catch(Exception e)
            {
                SuperController.LogError(EXCEPTION_PREFIX + e);
            }
        }
		
		void Update() {

		}

		// **************************
		// Functions
		// **************************
		protected bool IsDefaultLabelName(string val)
		{
			return val == DEFAULT_PRESET_LABEL;
		}
		
		protected void UpdateLightsPresetChooser()
		{
			lightsPresetsChoices = new List<String>();
			lightsPresetsChoices.Add("");
			foreach (string file in FileManagerSecure.GetFiles(PRESETS_LIGHTS_PATH))
			{
				if (file.Contains(".jpg")) continue;
				string presetName = PathExt.GetFileNameWithoutExtension(file);
				lightsPresetsChoices.Add(presetName);
			}
			LightsPresetChooser.choices = lightsPresetsChoices;
		}
		
		protected void UpdateWinCamPresetChooser()
		{
			wincamPresetsChoices = new List<String>();
			wincamPresetsChoices.Add("");
			foreach (string file in FileManagerSecure.GetFiles(PRESETS_WINCAM_PATH))
			{
				if (file.Contains(".jpg")) continue;
				string presetName = PathExt.GetFileNameWithoutExtension(file);
				wincamPresetsChoices.Add(presetName);
			}
			WinCamPresetChooser.choices = wincamPresetsChoices;
		}
		
		protected void UpdatePosePresetChooser()
		{
			posePresetsChoices = new List<String>();
			posePresetsChoices.Add("");
			foreach (string file in FileManagerSecure.GetFiles(PRESETS_POSE_PATH))
			{
				if (file.Contains(".jpg")) continue;
				string presetName = PathExt.GetFileNameWithoutExtension(file);
				posePresetsChoices.Add(presetName);
			}
			PosePresetChooser.choices = posePresetsChoices;
		}
		
		protected void UpdateSetsPresetChooser()
		{
			setPresetsChoices = new List<String>();
			setPresetsChoices.Add("");
			foreach (string file in FileManagerSecure.GetFiles(PRESETS_SETS_PATH))
			{
				if (file.Contains(".jpg")) continue;
				string presetName = PathExt.GetFileNameWithoutExtension(file);
				setPresetsChoices.Add(presetName);
			}
			SetPresetChooser.choices = setPresetsChoices;
		}

		protected void UpdateAllChoosers()
		{
			UpdateLightsPresetChooser();
			UpdateWinCamPresetChooser();
			UpdatePosePresetChooser();
			UpdateSetsPresetChooser();
		}

		protected void UpdateEyeTargetCharChooser()
		{
			eyeTargetCharChoices = new List<String>();
			foreach (Atom at in SuperController.singleton.GetAtoms().Where(atom => atom.type == "Person"))
			{
				eyeTargetCharChoices.Add(at.name);
			}
			EyeTargetCharacter.choices = eyeTargetCharChoices;
		}

		protected string GetLightsPresetsPath( string saveName )
		{
			string savePath = PRESETS_LIGHTS_PATH + "\\" + saveName + ".json";
			return savePath;
		}

		protected string GetLightLocalPresetsPath( string saveName, string lightName )
		{
			string savePath = PRESETS_LIGHTSLOCAL_PATH + "\\" + saveName + "_" + lightName + ".json";
			return savePath;
		}
		
		protected string GetGlobalLightPresetsPath( string saveName )
		{
			string savePath = PRESETS_GLOBALLIGHT_PATH + "\\" + saveName + ".json";
			return savePath;
		}
		
		protected string GetWinCamPresetsPath( string saveName )
		{
			string savePath = PRESETS_WINCAM_PATH + "\\" + saveName + ".json";
			return savePath;
		}
		
		protected string GetPosesPresetsPath( string saveName )
		{
			string savePath = PRESETS_POSE_PATH + "\\" + saveName + ".json";
			return savePath;
		}
		
		protected string GetPoseBackupPath()
		{
			string backupPath = BACKUP_PATH + "\\LastPose.json";
			return backupPath;
		}

		// **** START LIGHT FUNC
		protected virtual IEnumerator GetLightRig()
		{
			// If the light rig wasn't here at load...
			if (LightRig == null)
			{
				// ... searching for it...
				LightRig = SuperController.singleton.GetAtomByUid(LIGHTS_PARENT_NAME);
				if (LightRig == null)
				{
					// ... or creating it.
					yield return SuperController.singleton.AddAtomByType("Empty", LIGHTS_PARENT_NAME);
					LightRig = SuperController.singleton.GetAtomByUid(LIGHTS_PARENT_NAME);					
				}
			}
		}

		protected void SaveLightsPreset()
		{
			if (IsDefaultLabelName(PresetName.val))
			{
				SuperController.singleton.Alert("<b><color=green>"+PLUGIN_NAME+"</color></b>\nPlease choose a preset name before saving.", () => {});
				return;
			}

			JSONClass LightPreset = new JSONClass();
			bool lightFound = false;
			string fileName = PresetName.val;
			int lightCount = 1;
			
			foreach(Atom at in SuperController.singleton.GetAtoms().Where(atom => atom.type == "InvisibleLight") )
			{
				// Parenting to "nothing" (it prevents the "subscene" bug with the weird atom name)
				at.SetParentAtom("None");
				
				// Setting light name
				at.SetUID(LIGHTS_PREFIX + (lightCount < 10 ? "0" : "") + lightCount);

				// Saving the position
				JSONClass LightItem = new JSONClass();
				foreach (string st in at.GetStorableIDs())
				{
					if( !ignoreStorablesLights.Contains(st) ) LightItem[st] = at.GetStorableByID(st).GetJSON(true,true,true);
				}

				LightPreset[at.name] = LightItem;
				lightFound = true;

				// Saving the local preset (light settings, plugin)
				SaveTargetLightLocalPreset(at, fileName);
				
				// And parenting
				at.SetParentAtom(LightRig);
				lightCount++;
			}

			if (lightFound == true)
			{
				string lightsBackupPath = GetLightsPresetsPath( fileName );
				SuperController.singleton.SaveJSON(LightPreset, lightsBackupPath);

				string sceneLightbackupPath = GetGlobalLightPresetsPath( fileName );
				JSONClass globalLighting = SuperController.singleton.GetAtomByUid("CoreControl").GetStorableByID("GlobalLighting").GetJSON();
				SuperController.singleton.SaveJSON(globalLighting, sceneLightbackupPath);
				
				// Updating chooser
				UpdateLightsPresetChooser();
				LightsPresetChooser.valNoCallback = fileName;
				
				bool currentPreviewState = GetPreviewState();
				TogglePreviewState(false);
				
				SuperController.singleton.DoSaveScreenshot(lightsBackupPath.Replace(".json", ".jpg"), (val) =>
				{
					TogglePreviewState(currentPreviewState);
					SuperController.singleton.Alert("<b><color=green>"+PLUGIN_NAME+"</color></b>\nLights preset saved as:\n" + fileName, () => {});
				});
			}
			else
			{
				SuperController.singleton.Alert("<b><color=green>"+PLUGIN_NAME+"</color></b>\nNo lights found in your scene.", () => {});
			}
		}

		protected void DeleteLightsPreset()
		{
			string presetName = LightsPresetChooser.val;
			
			// Deleting the base light preset and preview and global light preset
			string presetPath = PRESETS_LIGHTS_PATH + "\\" + presetName + ".json";
			string presetPreviewPath = PRESETS_LIGHTS_PATH + "\\" + presetName + ".jpg";
			string presetGlobalPath = PRESETS_GLOBALLIGHT_PATH + "\\" + presetName + ".json";
			if (FileManagerSecure.FileExists(presetPath)) FileManagerSecure.DeleteFile(presetPath);
			if (FileManagerSecure.FileExists(presetPreviewPath)) FileManagerSecure.DeleteFile(presetPreviewPath);
			if (FileManagerSecure.FileExists(presetGlobalPath)) FileManagerSecure.DeleteFile(presetGlobalPath);
			
			// Deleting local lights presets
			foreach (string presetLocalFile in FileManagerSecure.GetFiles(PRESETS_LIGHTSLOCAL_PATH))
			{
				string fileName = PathExt.GetFileNameWithoutExtension(presetLocalFile);
				if (fileName.IndexOf(presetName + "_") == 0)
				{
					FileManagerSecure.DeleteFile(presetLocalFile);
				}
			}
			
			// Updating the chooser
			LightsPresetChooser.valNoCallback = "";
			UpdateLightsPresetChooser();
		}
		
		protected void SaveTargetLightLocalPreset(Atom at, string localPresetName)
		{
			JSONClass lightLocalPreset = GetLightJSONFromAtom(at);
			SuperController.singleton.SaveJSON(lightLocalPreset, PRESETS_LIGHTSLOCAL_PATH + "\\" + localPresetName + "_" + at.name + ".json");
		}
		
		protected void ClearLightAtoms()
		{
			foreach(Atom at in SuperController.singleton.GetAtoms().Where(atom => atom.type == "InvisibleLight") )
			{
				at.Remove();
			}
		}
		
		protected virtual IEnumerator LoadLightsPreset(string name, bool isSyncedLoading = false)
		{
			ClearLightAtoms();
			
			StartCoroutine("GetLightRig");
			LightRig.mainController.MoveControl( new Vector3(0f,0f,0f) );
			LightRig.mainController.RotateControl( new Vector3(0f,0f,0f) );
			
			// Every light
			JSONClass LoadedPreset = SuperController.singleton.LoadJSON(PRESETS_LIGHTS_PATH + "\\" + name + ".json") as JSONClass;
			foreach( KeyValuePair<string, JSONNode> item in LoadedPreset)
			{
				// Every storable for each light
				yield return SuperController.singleton.AddAtomByType("InvisibleLight", item.Key);
				Atom newLight = SuperController.singleton.GetAtomByUid(item.Key);
				newLight.SetParentAtom(LightRig);
				
				JSONClass LightStorables = item.Value as JSONClass;

				foreach (KeyValuePair<string, JSONNode> jsonData in LightStorables)
				{
					JSONClass StorableDatas = jsonData.Value as JSONClass;
					if (jsonData.Key.IndexOf("plugin#") == 0) continue;

					newLight.GetStorableByID(jsonData.Key).RestoreFromJSON( StorableDatas, true, true, null, true);
				}

				if (FileManagerSecure.FileExists(GetLightLocalPresetsPath(name, item.Key)))
				{
					JSONClass LoadedLocalPreset = SuperController.singleton.LoadJSON(GetLightLocalPresetsPath(name, newLight.name)) as JSONClass;
					ApplyLight(newLight, LoadedLocalPreset);
				}
			}
			
			// Global Lighting
			string globalLightFilename = PRESETS_GLOBALLIGHT_PATH + "\\" + name + ".json";
			if (FileManagerSecure.FileExists(globalLightFilename))
			{
				JSONClass LoadedGlobalPreset = SuperController.singleton.LoadJSON(globalLightFilename) as JSONClass;
				SuperController.singleton.GetAtomByUid("CoreControl").GetStorableByID("GlobalLighting").RestoreFromJSON( LoadedGlobalPreset, true, true, null, true);				
			}
			
			// Setting the proper exposure
			WinCamExposure.valNoCallback = GetWinCamExposure();

			// Resetting light rotation sliders
			LightRigRotationX.valNoCallback = 0f;
			LightRigRotationY.valNoCallback = 0f;
			LightRigRotationZ.valNoCallback = 0f;
			
			// Cascading other loading if needed
			if (!isSyncedLoading)
			{
				PresetNameInputField.text = name; // Applying label name only once
				
				if (SyncWinCamPresetLoading.val == true && WinCamPresetChooser.choices.Contains(name))
				{
					LoadWinCamPreset(name, true);
					WinCamPresetChooser.valNoCallback = name;
				}
				
				if (SyncPosePresetLoading.val == true && PosePresetChooser.choices.Contains(name))
				{
					LoadPosePreset(name, true);
					PosePresetChooser.valNoCallback = name;
				}
				
				if (SyncSetPresetLoading.val == true && SetPresetChooser.choices.Contains(name))
				{
					StartCoroutine(LoadSetPreset(name, true));
					SetPresetChooser.valNoCallback = name;
				}
			}
		}
		// **** END LIGHT FUNC
		
		// **** START POSE FUNC
		protected void SavePosePreset(bool isBackup = false)
		{
			string fileName = "";
			string backupPath = "";

			if( !isBackup ) {
				if (IsDefaultLabelName(PresetName.val))
				{
					SuperController.singleton.Alert("<b><color=green>"+PLUGIN_NAME+"</color></b>\nPlease choose a preset name before saving.", () => {});
					return;
				}
				
				fileName = PresetName.val;
				backupPath = GetPosesPresetsPath( fileName );
			}
			else
			{
				backupPath = GetPoseBackupPath();
			}
			JSONClass allPosesJSON = new JSONClass();
			int personCount = 0;
			
			foreach ( Atom at in SuperController.singleton.GetAtoms().Where(atom => atom.type == "Person") )
			{
				// Setting char name
				at.SetUID(CHARS_PREFIX + (personCount < 10 ? "0" : "") + personCount);
				
				JSONClass poseJSON = GetPoseJSONFromAtom(at);
				allPosesJSON[at.name] = poseJSON;
				personCount++;
			}

			if (personCount > 0)
			{
				SuperController.singleton.SaveJSON( allPosesJSON, backupPath);
				
				if( !isBackup ) {
					UpdatePosePresetChooser();
					PosePresetChooser.valNoCallback = fileName;
					
					bool currentPreviewState = GetPreviewState();
					TogglePreviewState(false);
					
					SuperController.singleton.DoSaveScreenshot( backupPath.Replace(".json", ".jpg"), (val) =>
					{
						TogglePreviewState(currentPreviewState);
						SuperController.singleton.Alert("<b><color=green>"+PLUGIN_NAME+"</color></b>\nPose preset saved as:\n" + fileName, () => {});						
					});
				}
			}
		}
		
		protected void LoadPosePreset(string name, bool isSyncedLoading = false)
		{
			// If we're in backup loading mode
			bool isLoadingBackup = name == "|backup|";
			
			JSONClass loadedPosePreset;
			if ( isLoadingBackup )
			{
				loadedPosePreset = SuperController.singleton.LoadJSON(GetPoseBackupPath()) as JSONClass;
			}
			else
			{
				// If we're loading an existing pose, let's backup the current one first
				SavePosePreset(true);
				
				loadedPosePreset = SuperController.singleton.LoadJSON(PRESETS_POSE_PATH + "\\" + name + ".json") as JSONClass;				
			}
			
			int personCount = 0;
			bool notEnoughPresets = false;
			Vector3 lastPosition = new Vector3();
			
			// Grabing persons in the scene. If there is less person than in the save file, it will simply get ignored
			foreach ( Atom at in SuperController.singleton.GetAtoms().Where(atom => atom.type == "Person") )
			{
				// If we have less presets than there is persons in the scene, moving the other characters a bit further
				if (loadedPosePreset[personCount] == null)
				{
					// Updating last position spot
					lastPosition.x = lastPosition.x + 1f;
					
					// Moving and loading a default pose for every character that is not in the preset
					// Also disabling collisions
					at.SetFreezePhysics(true);
					at.mainController.MoveControl( lastPosition );
					at.ResetPhysics(true,false);
					at.SetCollisionEnabled(false);
					
					ApplyPose(at, defaultInactivePoseJSON);
				}
				else
				{
					// Loading pose for current character
					JSONClass selectedPose = loadedPosePreset[personCount] as JSONClass;
					// Ensuring freeze and collisions are active
					at.SetFreezePhysics(false);
					at.SetCollisionEnabled(true);
					
					ApplyPose(at, selectedPose);
					lastPosition = at.mainController.control.position;
					personCount++;
				}
			}
			
			// Cascading other loading if needed
			if (!isSyncedLoading && !isLoadingBackup)
			{
				PresetNameInputField.text = name; // Applying label name only once
				
				if (SyncLightPresetLoading.val == true && LightsPresetChooser.choices.Contains(name))
				{
					StartCoroutine(LoadLightsPreset(name, true));
					LightsPresetChooser.valNoCallback = name;
				}
				
				if (SyncWinCamPresetLoading.val == true && WinCamPresetChooser.choices.Contains(name))
				{
					LoadWinCamPreset(name, true);
					WinCamPresetChooser.valNoCallback = name;
				}
				
				if (SyncSetPresetLoading.val == true && SetPresetChooser.choices.Contains(name))
				{
					StartCoroutine(LoadSetPreset(name, true));
					SetPresetChooser.valNoCallback = name;
				}
			}
		}
		
		protected void DeletePosePreset()
		{
			string presetName = PosePresetChooser.val;
			
			// Deleting the base light preset and preview and global light preset
			string presetPath = PRESETS_POSE_PATH + "\\" + presetName + ".json";
			string presetPreviewPath = PRESETS_POSE_PATH + "\\" + presetName + ".jpg";
			if (FileManagerSecure.FileExists(presetPath)) FileManagerSecure.DeleteFile(presetPath);
			if (FileManagerSecure.FileExists(presetPreviewPath)) FileManagerSecure.DeleteFile(presetPreviewPath);
			
			// Updating the chooser
			PosePresetChooser.valNoCallback = "";
			UpdatePosePresetChooser();
		}
		// **** END POSE FUNC
		
		// **** START WINDOW CAMERA FUNC
		protected void SaveWinCamPreset()
		{
			if (IsDefaultLabelName(PresetName.val))
			{
				SuperController.singleton.Alert("<b><color=green>"+PLUGIN_NAME+"</color></b>\nPlease choose a preset name before saving.", () => {});
				return;
			}
			
			JSONClass WCPreset = new JSONClass();
			foreach (string st in WinCamAtom.GetStorableIDs())
			{
				
				if( !ignoreStorablesWinCam.Contains(st) ) WCPreset[st] = WinCamAtom.GetStorableByID(st).GetJSON();
			}

			string fileName = PresetName.val;
			string backupPath = GetWinCamPresetsPath( fileName );
			SuperController.singleton.SaveJSON( WCPreset, backupPath);
			
			// Updating chooser
			UpdateWinCamPresetChooser();
			WinCamPresetChooser.valNoCallback = fileName;
			SuperController.singleton.Alert("<b><color=green>"+PLUGIN_NAME+"</color></b>\nWindow Camera preset saved as:\n" + fileName, () => {});
		}
		
		protected void LoadWinCamPreset(string name, bool isSyncedLoading = false)
		{
			JSONClass LoadedPreset = SuperController.singleton.LoadJSON(PRESETS_WINCAM_PATH + "\\" + name + ".json") as JSONClass;
			foreach (KeyValuePair<string, JSONNode> jsonData in LoadedPreset)
			{
				// We ignore the display control parameters to avoid changing it's position
				if (jsonData.Key == "DisplayControl") continue;
				
				JSONClass StorableDatas = jsonData.Value as JSONClass;
				
				// Overriding some values of the camera control to the current state
				if (jsonData.Key == "CameraControl")
				{
					StorableDatas["showHUDView"] = "" + GetPreviewState();
					StorableDatas["cameraOn"] = "" + WinCamControl.GetBoolParamValue("cameraOn");
					WinCamFOV.valNoCallback = float.Parse(StorableDatas["FOV"].Value);
				}
				
				WinCamAtom.GetStorableByID(jsonData.Key).RestoreFromJSON(StorableDatas, true, true, null, true);
			}
			
			// Cascading other loading if needed
			if (!isSyncedLoading)
			{
				PresetNameInputField.text = name; // Applying label name only once
				
				if (SyncLightPresetLoading.val == true && LightsPresetChooser.choices.Contains(name))
				{
					StartCoroutine(LoadLightsPreset(name, true));
					LightsPresetChooser.valNoCallback = name;
				}
				
				if (SyncPosePresetLoading.val == true && PosePresetChooser.choices.Contains(name))
				{
					LoadPosePreset(name, true);
					PosePresetChooser.valNoCallback = name;
				}
				
				if (SyncSetPresetLoading.val == true && SetPresetChooser.choices.Contains(name))
				{
					StartCoroutine(LoadSetPreset(name, true));
					SetPresetChooser.valNoCallback = name;
				}
			}
		}
		
		protected void DeleteWinCamPreset()
		{
			string presetName = WinCamPresetChooser.val;
			
			// Deleting the base light preset and preview and global light preset
			string presetPath = PRESETS_WINCAM_PATH + "\\" + presetName + ".json";
			if (FileManagerSecure.FileExists(presetPath)) FileManagerSecure.DeleteFile(presetPath);

			// Updating the chooser
			WinCamPresetChooser.valNoCallback = "";
			UpdateWinCamPresetChooser();
		}
		// **** END WINDOW CAMERA FUNC
		
		// **** START SETS FUNC
		protected void ClearSetSubscene()
		{
			foreach(Atom at in SuperController.singleton.GetAtoms().Where(atom => atom.type == "SubScene") )
			{
				if (at.name == "STMA_Set")
				{
					at.Remove();
				}
			}
		}
		
		protected virtual IEnumerator LoadSetPreset(string name, bool isSyncedLoading = false)
		{
			Atom setAtom = SuperController.singleton.GetAtomByUid("STMA_Set");
			if (setAtom == null)
			{
				yield return SuperController.singleton.AddAtomByType("SubScene", "STMA_Set");
				setAtom = SuperController.singleton.GetAtomByUid("STMA_Set");
			}

			// Creating my base json for the load process
			JSONClass baseSubscene = new JSONClass();
			baseSubscene["id"] = "Subscene";
			baseSubscene["storePath"] = "Custom/SubScene/StudioMaestro/STMA_Set/" + name + ".json";

			setAtom.GetStorableByID("SubScene").RestoreFromJSON( baseSubscene, true, true, null, true);
			SubScene tmpSubscene = setAtom.GetStorableByID("SubScene") as SubScene;
			
			tmpSubscene.LoadSubScene();
			
			// Cascading other loading if needed
			if (!isSyncedLoading)
			{
				PresetNameInputField.text = name; // Applying label name only once
				
				if (SyncLightPresetLoading.val == true && LightsPresetChooser.choices.Contains(name))
				{
					StartCoroutine(LoadLightsPreset(name, true));
					LightsPresetChooser.valNoCallback = name;
				}
				
				if (SyncWinCamPresetLoading.val == true && WinCamPresetChooser.choices.Contains(name))
				{
					LoadWinCamPreset(name, true);
					WinCamPresetChooser.valNoCallback = name;
				}
				
				if (SyncPosePresetLoading.val == true && PosePresetChooser.choices.Contains(name))
				{
					LoadPosePreset(name, true);
					PosePresetChooser.valNoCallback = name;
				}
				
			}
		}
		// **** END SETS FUNC

		// *** Start - Thanks Acid (Timeline 283)
		// Obviously, some methods names renamed to my own tastes... sue me.
		private JSONClass GetLightJSONFromAtom(Atom atom, bool includeRoot = true, bool includePose = true, bool includeMorphs = true, bool useMergeLoad = false)
		{
			if (atom.type != "InvisibleLight") return null;

			var storables = atom.GetStorableIDs()
				.Select(atom.GetStorableByID);
			var storablesJSON = new JSONArray();
			foreach (var storable in storables)
			{
				var storableJSON = storable.GetJSON(true, true, true);
				if( !ignoreStorablesLightLocal.Contains(storableJSON["id"] ) ) storablesJSON.Add(storableJSON);
			}
			
			JSONClass lightJSON = new JSONClass
			{
				["setUnlistedParamsToDefault"] = { AsBool = true },
				["storables"] = storablesJSON
			};
			
			return lightJSON;
		}
		
		private JSONClass GetPoseJSONFromAtom(Atom atom, bool includeRoot = true, bool includePose = true, bool includeMorphs = true, bool useMergeLoad = false)
		{
			if (atom.type != "Person") return null;

			var storables = atom.GetStorableIDs()
				.Select(atom.GetStorableByID)
				.Where(t => !t.exclude && t.gameObject.activeInHierarchy)
				.Where(t => ShouldStorableBeIncluded(t, includeRoot, includePose, includeMorphs));
			var storablesJSON = new JSONArray();
			foreach (var storable in storables)
			{
				var storableJSON = storable.GetJSON(true, false, storable.name != "geometry");
				storablesJSON.Add(storableJSON);
			}

			JSONClass poseJSON = new JSONClass
			{
				["poseSettings"] = new JSONClass
				{
					["includeRoot"] = { AsBool = includeRoot },
					["includePose"] = { AsBool = includePose },
					["includeMorphs"] = { AsBool = includeMorphs },
					["useMergeLoad"] = { AsBool = useMergeLoad },
				},
				["setUnlistedParamsToDefault"] = { AsBool = true },
				["V2"] = { AsBool = true },
				["storables"] = storablesJSON
			};

			return poseJSON;
		}
		
		private bool ShouldStorableBeIncluded(JSONStorable t, bool includeRoot, bool includePose, bool includeMorphs)
		{
			if (includePose && t is FreeControllerV3) return includeRoot || ((FreeControllerV3) t).name != "control";
			if (includePose && t is DAZBone) return true;
			if (includeMorphs && t.storeId == "geometry" && t is DAZCharacterSelector) return true;
			return false;
		}
		
		private void ApplyPose(Atom at, JSONClass poseJSON)
		{
			var posePresetsManagerControls = at.presetManagerControls.First(pmc => pmc.name == "PosePresets");
			var posePresetsManager = posePresetsManagerControls.GetComponent<PresetManager>();
			posePresetsManager.LoadPresetFromJSON(poseJSON, false);
		}
		
		private void ApplyLight(Atom at, JSONClass lightJSON)
		{
			var lightPresetsManagerControls = at.mainPresetControl;
			var lightPresetsManager = lightPresetsManagerControls.GetComponent<PresetManager>();
			lightPresetsManager.LoadPresetFromJSON(lightJSON, false);

			// I don't know why the fuck that preset system does not restore the proper plugin settings
			// so let's do it myself :x
			// you might think I could remove the preset loading system, but it is actually used to properly initialize the plugins
			// without that, I haven't found a workaround that allows you to create the plugins instances yourself
			for (int i = 0; i < lightJSON["storables"].Count; i++)
			{
				string stID = lightJSON["storables"][i]["id"];
				if (stID.IndexOf("plugin#") == 0)
				{
					JSONClass tmpJSON = lightJSON["storables"][i] as JSONClass;
					at.GetStorableByID(lightJSON["storables"][i]["id"]).RestoreFromJSON( tmpJSON, true, true, null, true);
				}
			}
		}
		// *** End - Thanks Acid (Timeline 283)

		private void GrabWinCamPreviewComponents()
		{
			WinCamControl = WinCamAtom?.GetStorableByID("CameraControl") as CameraControl;

			WinCamPreviewBackground = WinCamControl.HUDView.transform.Find("Background");
			WinCamPreviewImage = WinCamControl.HUDView.transform.Find("PlaneForCamera");
		}

		private void ToggleWinCam()
		{
			bool camState = !WinCamControl.GetBoolParamValue("cameraOn");
			WinCamControl.SetBoolParamValue("cameraOn", camState);
			if( camState == false ) TogglePreviewState(camState);
		}
		
		private void ToggleWinCamState(bool state)
		{
			WinCamControl.SetBoolParamValue("cameraOn", state);
		}
		
		private void TogglePreview()
		{
			WinCamControl.showHUDView = !WinCamControl.showHUDView;

			if (WinCamControl.showHUDView == true)
			{
				ToggleWinCamState(true);
				SetPreviewAspectRatio();
			}
		}

		private void TogglePreviewState(bool state)
		{
			WinCamControl.showHUDView = state;
		}

		private bool GetPreviewState()
		{
			return WinCamControl.showHUDView;
		}
		
		private void FlipPreviewAspectRatio()
		{
			previewIsPortrait = !previewIsPortrait;
			SetPreviewAspectRatio();
		}

		private void SetPreviewAspectRatio()
		{
			switch (previewIsPortrait)
			{
				case false:
					SetPreviewAsPortrait();
					break;
				default:
					SetPreviewAsLandscape();
					break;
			}			
		}
		
		private void SetPreviewAsPortrait()
		{
			WinCamPreviewImage.localScale = new Vector3(0.05f, 1f, 0.0281f);
			WinCamPreviewImage.localPosition = new Vector3(-0.2f, 0.5f, 0f);
			WinCamPreviewImage.localRotation = Quaternion.Euler(180f, -90, -90);
			
			WinCamPreviewBackground.localScale = new Vector3(0.052f, 1f, 0.03f);
			WinCamPreviewBackground.localPosition = new Vector3(-0.202f, 0.5f, -0.01f);
			WinCamPreviewBackground.localRotation = Quaternion.Euler(180f, -90f, -90f);
		}
		
		private void SetPreviewAsLandscape()
		{
			WinCamPreviewImage.localScale = new Vector3(0.05f, 1f, 0.0281f);
			WinCamPreviewImage.localPosition = new Vector3(-0.2f, 0.5f, 0f);
			WinCamPreviewImage.localRotation = Quaternion.Euler(90f, 0, 0);
			
			WinCamPreviewBackground.localScale = new Vector3(0.052f, 1f, 0.03f);
			WinCamPreviewBackground.localPosition = new Vector3(-0.202f, 0.5f, -0.01f);
			WinCamPreviewBackground.localRotation = Quaternion.Euler(90f, 0, 0);
		}

		protected void InitSuperShotEmptyAtom()
		{
			bool emptyMissing = false;
			Atom supershotAtom = SuperController.singleton.GetAtomByUid("STMA_SuperShot");
			
			// No empty with SuperShot? Creating it
			if (supershotAtom == null)
			{
				emptyMissing = true;
				StartCoroutine(LoadSuperShotDefaults());
			}

			// If we did find an empty checking for supershot setup
			if (!emptyMissing)
			{
				if (!supershotAtom.GetStorableByID("plugin#0_MacGruber.SuperShot"))
				{
					SuperController.singleton.Alert(TXT_ERROR_SUPERSHOT,
						delegate
						{
							supershotAtom.Remove();
							StartCoroutine(LoadSuperShotDefaults());
						}, () => { });
				}
			}
		}

		protected virtual IEnumerator LoadSuperShotDefaults()
		{
			yield return SuperController.singleton.AddAtomByType("Empty", "STMA_SuperShot");
			Atom supershotAtom = SuperController.singleton.GetAtomByUid("STMA_SuperShot");

			JSONClass emptyDefaultPreset = SuperController.singleton.LoadJSON(DEFAULT_SS_EMPTY_PATH) as JSONClass;
				
			var emptyPresetsManagerControls = supershotAtom.mainPresetControl;
			var emptyPresetsManager = emptyPresetsManagerControls.GetComponent<PresetManager>();
			emptyPresetsManager.LoadPresetFromJSON(emptyDefaultPreset, false);
				
			// again, don't know why the thing does not restore properly :/
			for (int i = 0; i < emptyDefaultPreset["storables"].Count; i++)
			{
				string stID = emptyDefaultPreset["storables"][i]["id"];
				if (stID.IndexOf("plugin#") == 0)
				{
					JSONClass tmpJSON = emptyDefaultPreset["storables"][i] as JSONClass;
					supershotAtom.GetStorableByID(emptyDefaultPreset["storables"][i]["id"]).RestoreFromJSON( tmpJSON, true, true, null, true);
				}
			}
		}
		
		protected void SaveSuperShotScreenshot()
		{
			JSONClass fakeTriggerData = new JSONClass();
			fakeTriggerData["id"] = this.name;
			fakeTriggerData["Load_FakeTrigger"]["displayName"] = "A_TakeScreenshotWindowCamera";
			fakeTriggerData["Load_FakeTrigger"]["startActions"][0]["name"] = "A_TakeScreenshotWindowCamera";
			fakeTriggerData["Load_FakeTrigger"]["startActions"][0]["receiverAtom"] = "STMA_SuperShot";
			fakeTriggerData["Load_FakeTrigger"]["startActions"][0]["receiver"] = "plugin#0_MacGruber.SuperShot";
			fakeTriggerData["Load_FakeTrigger"]["startActions"][0]["receiverTargetName"] = "TakeScreenshotWindowCamera";

			Load_FakeTrigger.RestoreFromJSON( fakeTriggerData, subScenePrefix, mergeRestore, true );

			Load_FakeTrigger.Trigger();
		}

		protected void ChangeWinCamFOV( float fovValue )
		{
			WinCamControl.SetFloatParamValue("FOV", fovValue);
		}

		protected float GetWinCamExposure()
		{
			return SuperController.singleton.GetAtomByUid("CoreControl").GetStorableByID("GlobalLighting").GetFloatParamValue("camExposure");
		}
		
		protected void ChangeWinCamExposure( float expVal )
		{
			SuperController.singleton.GetAtomByUid("CoreControl").GetStorableByID("GlobalLighting").SetFloatParamValue("camExposure", expVal);
		}

		protected void TweakCamPosition(Vector3 addVector)
		{
			WinCamAtom.mainController.MoveControl( WinCamAtom.mainController.control.position + addVector );
		}
		
		protected void TweakCamRotation(Vector3 addVector)
		{
			WinCamAtom.mainController.RotateControl( WinCamAtom.mainController.control.localRotation.eulerAngles + addVector );
		}

		protected float GetTweakCamRotationValue()
		{
			float rotVal = WinCamTweakVal.val < 0.1 ? WinCamTweakVal.val * 20f : WinCamTweakVal.val * 10f;
			return rotVal;
		}
		
		protected bool IsEyeTargetSelected()
		{
			return charEyeTarget != null;
		}

		protected void AlertEyeTargetNoSelection()
		{
			SuperController.singleton.Alert(TXT_WARNING_NOEYETARGETSELECTION, () => {});
		}
		
		protected void TweakEyeTargetPosition(Vector3 addVector)
		{
			charEyeTarget.MoveControl( charEyeTarget.control.position + addVector );
		}
		
		protected void MoveEyeTargetPosition(Vector3 newVector)
		{
			charEyeTarget.MoveControl( newVector );
		}
		
		protected void SaveBrowserDefaults()
		{
			defaultBrowserSettings = new JSONClass();
			defaultBrowserSettings["defaultPath"] = SuperController.singleton.fileBrowserUI.defaultPath;
			defaultBrowserSettings["fileFormat"] = SuperController.singleton.fileBrowserUI.fileFormat;
			defaultBrowserSettings["showFiles"] = "" + SuperController.singleton.fileBrowserUI.showFiles;
			defaultBrowserSettings["showDirs"] = "" + SuperController.singleton.fileBrowserUI.showDirs;
			defaultBrowserSettings["hideExtension"] = "" + SuperController.singleton.fileBrowserUI.hideExtension;
			defaultBrowserSettings["showInstallFolderInDirectoryList"] = "" + SuperController.singleton.fileBrowserUI.showInstallFolderInDirectoryList;
		}
		
		protected void RestoreBrowserDefaults()
		{
			SuperController.singleton.fileBrowserUI.defaultPath = defaultBrowserSettings["defaultPath"];
			SuperController.singleton.fileBrowserUI.fileFormat = defaultBrowserSettings["fileFormat"];
			SuperController.singleton.fileBrowserUI.showFiles = bool.Parse(defaultBrowserSettings["showFiles"]);
			SuperController.singleton.fileBrowserUI.showDirs = bool.Parse(defaultBrowserSettings["showDirs"]);
			SuperController.singleton.fileBrowserUI.hideExtension = bool.Parse(defaultBrowserSettings["hideExtension"]);
			SuperController.singleton.fileBrowserUI.showInstallFolderInDirectoryList = bool.Parse(defaultBrowserSettings["showInstallFolderInDirectoryList"]);
		}
		
		// **************************
		// Callbacks & similar
		// **************************
		protected void OnPresetNameChanged()
		{
			if (PresetNameInputField.text == DEFAULT_PRESET_LABEL) return;
			
			string newName = "";
			if (PresetNameInputField.text == "")
			{
				newName = DEFAULT_PRESET_LABEL;
			}
			else
			{
				newName = Utils.CleanString(PresetNameInputField.text);
			}
			PresetNameInputField.text = newName;
			PresetName.val = newName;
		}
		
		private void OnSaveLightsPreset()
		{
			string fileName = PresetName.val;
			string lightsBackupPath = GetLightsPresetsPath( fileName );
			
			if( FileManagerSecure.FileExists(lightsBackupPath) )
			{
				SuperController.singleton.Alert("Do you want to overwrite this light preset\n<b>" + fileName + "</b>?", delegate { SaveLightsPreset(); }, () => {});
			}
			else
			{
				SaveLightsPreset();
			}
		}

		private void OnDeleteLightsPreset()
		{
			string fileName = LightsPresetChooser.val;
			if (fileName == "") return;
			SuperController.singleton.Alert("Are you sure you want to delete the light preset: \n<b>" + fileName + "</b>?", delegate { DeleteLightsPreset(); }, () => {});
		}
		
		private void OnSaveWinCamPreset()
		{
			string fileName = PresetName.val;
			string wincamBackupPath = GetWinCamPresetsPath( fileName );
			
			if( FileManagerSecure.FileExists(wincamBackupPath) )
			{
				SuperController.singleton.Alert("Do you want to overwrite this\nwindow camera preset:\n<b>" + fileName + "</b>?", delegate { SaveWinCamPreset(); }, () => {});
			}
			else
			{
				SaveWinCamPreset();
			}
		}

		private void OnDeleteWinCamPreset()
		{
			string fileName = WinCamPresetChooser.val;
			if (fileName == "") return;
			SuperController.singleton.Alert("Are you sure you want to delete the window camera preset: \n<b>" + fileName + "</b>?", delegate { DeleteWinCamPreset(); }, () => {});
		}

		private void OnSavePosePreset()
		{
			string fileName = PresetName.val;
			string poseBackupPath = GetPosesPresetsPath( fileName );
			
			if( FileManagerSecure.FileExists(poseBackupPath) )
			{
				SuperController.singleton.Alert("Do you want to overwrite this pose preset\n<b>" + fileName + "</b>?", delegate { SavePosePreset(); }, () => {});
			}
			else
			{
				SavePosePreset();
			}
		}

		private void OnSelectPosePreset(string presetName)
		{
			if (presetName == "") return;
			LoadPosePreset(presetName);
		}
		
		private void OnDeletePosePreset()
		{
			string fileName = PosePresetChooser.val;
			if (fileName == "") return;
			SuperController.singleton.Alert("Are you sure you want to delete the pose preset: \n<b>" + fileName + "</b>?", delegate { DeletePosePreset(); }, () => {});
		}

		private void OnRestoreLastPoseSnapshot()
		{
			SuperController.singleton.Alert("Do you want to restore your previous pose snapshot?", delegate { LoadPosePreset("|backup|"); }, () => {});
		}

		private void OnSnapshotCurrentPose()
		{
			SavePosePreset(true);
		}

		private void OnSearchPose()
		{
			SuperController.singleton.fileBrowserUI.defaultPath = PRESETS_POSE_PATH;
			SuperController.singleton.fileBrowserUI.SetTitle("Select a pose");
			SuperController.singleton.fileBrowserUI.fileFormat = "json";
			SuperController.singleton.fileBrowserUI.showFiles = true;
			SuperController.singleton.fileBrowserUI.showDirs = false;
			SuperController.singleton.fileBrowserUI.SetTextEntry(false);
			SuperController.singleton.fileBrowserUI.hideExtension = false;
			SuperController.singleton.fileBrowserUI.showInstallFolderInDirectoryList = false;
			
			List<ShortCut> shortCutList = new List<ShortCut>();
			ShortCut shortCutSaves = new ShortCut();
			shortCutSaves.path = PRESETS_POSE_PATH;
			shortCutSaves.displayName = "Pose Presets";
			shortCutSaves.flatten = false;
			shortCutList.Add(shortCutSaves);
			SuperController.singleton.fileBrowserUI.shortCuts = shortCutList;
			
			SuperController.singleton.fileBrowserUI.Show((val) =>
			{
				if (val != "")
				{
					LoadPosePreset(PathExt.GetFileNameWithoutExtension(val));
					PosePresetChooser.val = PathExt.GetFileNameWithoutExtension(val);
				}
			});

			RestoreBrowserDefaults();
		}

		private void OnSelectSetPreset(string presetName)
		{
			if (presetName == "") return;
			StartCoroutine(LoadSetPreset(presetName));
		}

		private void OnClearSet()
		{
			ClearSetSubscene();
			SetPresetChooser.valNoCallback = "";
		}

		private void OnRefreshSetList()
		{
			UpdateSetsPresetChooser();
		}

		private void OnSearchSet()
		{
			SuperController.singleton.fileBrowserUI.defaultPath = PRESETS_SETS_PATH;
			SuperController.singleton.fileBrowserUI.SetTitle("Select a set preset");
			SuperController.singleton.fileBrowserUI.fileFormat = "json";
			SuperController.singleton.fileBrowserUI.showFiles = true;
			SuperController.singleton.fileBrowserUI.showDirs = false;
			SuperController.singleton.fileBrowserUI.SetTextEntry(false);
			SuperController.singleton.fileBrowserUI.hideExtension = false;
			SuperController.singleton.fileBrowserUI.showInstallFolderInDirectoryList = false;
			
			List<ShortCut> shortCutList = new List<ShortCut>();
			ShortCut shortCutSaves = new ShortCut();
			shortCutSaves.path = PRESETS_SETS_PATH;
			shortCutSaves.displayName = "Sets Presets";
			shortCutSaves.flatten = false;
			shortCutList.Add(shortCutSaves);
			SuperController.singleton.fileBrowserUI.shortCuts = shortCutList;
			
			SuperController.singleton.fileBrowserUI.Show((val) =>
			{
				if (val != "")
				{
					StartCoroutine(LoadSetPreset(PathExt.GetFileNameWithoutExtension(val)));
					WinCamPresetChooser.val = PathExt.GetFileNameWithoutExtension(val);
				}
			});
			
			RestoreBrowserDefaults();
		}

		private void OnSelectLightsPreset(string presetName)
		{
			if (presetName == "") return;
			StartCoroutine(LoadLightsPreset(presetName));
		}
		
		private void OnSearchLight()
		{
			SuperController.singleton.fileBrowserUI.defaultPath = PRESETS_LIGHTS_PATH;
			SuperController.singleton.fileBrowserUI.SetTitle("Select a light preset");
			SuperController.singleton.fileBrowserUI.fileFormat = "json";
			SuperController.singleton.fileBrowserUI.showFiles = true;
			SuperController.singleton.fileBrowserUI.showDirs = false;
			SuperController.singleton.fileBrowserUI.SetTextEntry(false);
			SuperController.singleton.fileBrowserUI.hideExtension = false;
			SuperController.singleton.fileBrowserUI.showInstallFolderInDirectoryList = false;
			
			List<ShortCut> shortCutList = new List<ShortCut>();
			ShortCut shortCutSaves = new ShortCut();
			shortCutSaves.path = PRESETS_LIGHTS_PATH;
			shortCutSaves.displayName = "Lights Presets";
			shortCutSaves.flatten = false;
			shortCutList.Add(shortCutSaves);
			SuperController.singleton.fileBrowserUI.shortCuts = shortCutList;
			
			SuperController.singleton.fileBrowserUI.Show((val) =>
			{
				if (val != "")
				{
					StartCoroutine(LoadLightsPreset(PathExt.GetFileNameWithoutExtension(val)));
					LightsPresetChooser.valNoCallback = PathExt.GetFileNameWithoutExtension(val);
				}
			});
			
			RestoreBrowserDefaults();
		}

		private void OnSelectWinCamPreset(string presetName)
		{
			if (presetName == "") return;
			LoadWinCamPreset(presetName);
		}

		private void OnRefreshChoosers()
		{
			UpdateAllChoosers();
		}

		private void OnRotateLightRig()
		{
			if (LightRig != null)
			{
				LightRig.mainController.RotateControl( new Vector3(LightRigRotationX.val,LightRigRotationY.val,LightRigRotationZ.val) );				
			}
		}

		private void OnChangeWinCamFOV( float val )
		{
			ChangeWinCamFOV(val);
		}

		private void OnChangeWinCamExposure(float val)
		{
			ChangeWinCamExposure(val);
		}

		private void OnTogglePreview()
		{
			TogglePreview();
		}

		private void OnFlipPreviewAspectRatio()
		{
			FlipPreviewAspectRatio();
		}

		private void OnToggleWinCam()
		{
			ToggleWinCam();
		}

		private void OnSaveSuperShotScreenshot()
		{
			SaveSuperShotScreenshot();
		}

		// Camera tweaks
		private void OnTweakCamUp()
		{
			Vector3 addVecValue = new Vector3(0, WinCamTweakVal.val, 0);
			TweakCamPosition(addVecValue);
		}
		
		private void OnTweakCamDown()
		{
			Vector3 addVecValue = new Vector3(0, -WinCamTweakVal.val, 0);
			TweakCamPosition(addVecValue);
		}
		
		private void OnTweakCamLeft()
		{
			Vector3 addVecValue = new Vector3(WinCamTweakVal.val, 0, 0);
			TweakCamPosition(addVecValue);
		}
		
		private void OnTweakCamRight()
		{
			Vector3 addVecValue = new Vector3(-WinCamTweakVal.val, 0, 0);
			TweakCamPosition(addVecValue);
		}
		
		private void OnTweakCamForward()
		{
			Vector3 addVecValue = new Vector3(0, 0, -WinCamTweakVal.val);
			TweakCamPosition(addVecValue);
		}
		
		private void OnTweakCamBackward()
		{
			Vector3 addVecValue = new Vector3(0, 0, WinCamTweakVal.val);
			TweakCamPosition(addVecValue);
		}
		
		private void OnTweakCamRotZPos()
		{
			Vector3 addVecValue = new Vector3(0, 0, -GetTweakCamRotationValue());
			TweakCamRotation(addVecValue);
		}
		
		private void OnTweakCamRotZNeg()
		{
			Vector3 addVecValue = new Vector3(0, 0, GetTweakCamRotationValue());
			TweakCamRotation(addVecValue);
		}
		
		private void OnTweakCamRotYPos()
		{
			Vector3 addVecValue = new Vector3(0, -GetTweakCamRotationValue(), 0);
			TweakCamRotation(addVecValue);
		}
		
		private void OnTweakCamRotYNeg()
		{
			Vector3 addVecValue = new Vector3(0, GetTweakCamRotationValue(), 0);
			TweakCamRotation(addVecValue);
		}
		
		private void OnTweakCamRotXPos()
		{
			Vector3 addVecValue = new Vector3(-GetTweakCamRotationValue(), 0, 0);
			TweakCamRotation(addVecValue);
		}
		
		private void OnTweakCamRotXNeg()
		{
			Vector3 addVecValue = new Vector3(GetTweakCamRotationValue(), 0, 0);
			TweakCamRotation(addVecValue);
		}
		
		// Eye Target tweaks
		private void OnTweakETUp()
		{
			if (!IsEyeTargetSelected())
			{
				AlertEyeTargetNoSelection();
				return;
			}

			Vector3 addVecValue = new Vector3(0, EyeTargetTweakVal.val, 0);
			TweakEyeTargetPosition(addVecValue);
		}
		
		private void OnTweakETDown()
		{
			if (!IsEyeTargetSelected())
			{
				AlertEyeTargetNoSelection();
				return;
			}
			Vector3 addVecValue = new Vector3(0, -EyeTargetTweakVal.val, 0);
			TweakEyeTargetPosition(addVecValue);
		}
		
		private void OnTweakETLeft()
		{
			if (!IsEyeTargetSelected())
			{
				AlertEyeTargetNoSelection();
				return;
			}
			Vector3 addVecValue = new Vector3(EyeTargetTweakVal.val, 0, 0);
			TweakEyeTargetPosition(addVecValue);
		}
		
		private void OnTweakETRight()
		{
			if (!IsEyeTargetSelected())
			{
				AlertEyeTargetNoSelection();
				return;
			}
			Vector3 addVecValue = new Vector3(-EyeTargetTweakVal.val, 0, 0);
			TweakEyeTargetPosition(addVecValue);
		}
		
		private void OnTweakETForward()
		{
			if (!IsEyeTargetSelected())
			{
				AlertEyeTargetNoSelection();
				return;
			}
			Vector3 addVecValue = new Vector3(0, 0, -EyeTargetTweakVal.val);
			TweakEyeTargetPosition(addVecValue);
		}
		
		private void OnTweakETBackward()
		{
			if (!IsEyeTargetSelected())
			{
				AlertEyeTargetNoSelection();
				return;
			}
			Vector3 addVecValue = new Vector3(0, 0, EyeTargetTweakVal.val);
			TweakEyeTargetPosition(addVecValue);
		}

		private void OnChangeEyeTargetChar(string charName)
		{
			foreach (Atom at in SuperController.singleton.GetAtoms().Where(atom => atom.name == charName))
			{
				charEyeTarget = at.GetStorableByID("eyeTargetControl") as FreeControllerV3;
			}
		}

		protected void OnSetEyeTargetLookAt()
		{
			if (!IsEyeTargetSelected())
			{
				AlertEyeTargetNoSelection();
				return;
			}
			
			foreach (Atom at in SuperController.singleton.GetAtoms().Where(atom => atom.name == EyeTargetCharacter.val))
			{
				JSONStorable eyesSettings = at.GetStorableByID("Eyes");
				eyesSettings.SetStringChooserParamValue("lookMode", "Target");
			}
		}
		
		protected void OnMoveEyeTargetToWinCamera()
		{
			if (!IsEyeTargetSelected())
			{
				AlertEyeTargetNoSelection();
				return;
			}
			MoveEyeTargetPosition( WinCamAtom.mainController.control.position );
		}
		
		protected void OnMoveEyeTargetToMyPosition()
		{
			if (!IsEyeTargetSelected())
			{
				AlertEyeTargetNoSelection();
				return;
			}
			MoveEyeTargetPosition(SuperController.singleton.centerCameraTarget.transform.position);
		}
		
		protected void OnShowTabCamTweaks(int tabNum)
		{
			Color tab1BtnBg = tabNum == 1 ? tabTweaksSelBgColor : tabTweaksBgColor;
			Color tab2BtnBg = tabNum == 2 ? tabTweaksSelBgColor : tabTweaksBgColor;
			
			Utils.SetButtonColor(uiBtnCamTweaks, tab1BtnBg);
			Utils.SetButtonColor(uiBtnEyeTargetTweaks, tab2BtnBg);
			
			bool tab1Show = tabNum == 1 ? true : false;
			bool tab2Show = tabNum == 2 ? true : false;
			
			ToggleUIDynamicGrp(tabCamTweaksUIs, tab1Show);
			ToggleUIDynamicGrp(tabEyeTargetTweaksUIs, tab2Show);

			if (tab2Show)
			{
				UpdateEyeTargetCharChooser();
			}
		}
		
		// **************************
		// OVERRIDES SAVING/LOADING
		// **************************


		// **************************
		// HANDLERS
		// **************************
		private void OnSceneLoaded()
		{
			// Loading presets list for everything
			UpdateAllChoosers();
			
			// Grabbing atoms
			WinCamAtom = SuperController.singleton.GetAtomByUid("WindowCamera");
			StartCoroutine("GetLightRig");

			// Win camera components
			GrabWinCamPreviewComponents();
			
			// Testing for MacGruber Essentials 16
			if (!FileManagerSecure.FileExists("AddonPackages/MacGruber.Essentials.16.var"))
			{
				SuperController.singleton.Alert(TXT_ERROR_ESSENTIALS, () => {});
			}
				
			InitSuperShotEmptyAtom();
		}
		
		// **************************
		// Time to cleanup !
		// **************************
		void OnDestroy()
		{
			SuperController.singleton.onSceneLoadedHandlers -= OnSceneLoaded;
			
			if( Load_FakeTrigger != null ) Load_FakeTrigger.Remove();
			
			// Resetting the window camera preview to it's original state, if changed
			if (WinCamPreviewImage != null)
			{
				TogglePreviewState(false);
				
				WinCamPreviewImage.localScale = WinCamPreviewImgOriginalScale;
				WinCamPreviewImage.localPosition = WinCamPreviewImgOriginalPos;
				WinCamPreviewImage.localRotation = Quaternion.Euler( WinCamPreviewOriginalRot.x, WinCamPreviewOriginalRot.y, WinCamPreviewOriginalRot.z );
			
				WinCamPreviewBackground.localScale = WinCamPreviewBgOriginalScale;
				WinCamPreviewBackground.localPosition = WinCamPreviewBgOriginalPos;
				WinCamPreviewBackground.localRotation = Quaternion.Euler( WinCamPreviewOriginalRot.x, WinCamPreviewOriginalRot.y, WinCamPreviewOriginalRot.z );
			}
		}


		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}
		
		protected void ToggleUIDynamicGrp(List<UIDynamic> uiGrp,bool enabled)
		{
			foreach( UIDynamic uidElem in uiGrp )
			{
				ToggleUIDynamic(enabled, uidElem);
			}
		}
		
		protected void ToggleUIDynamic(bool enabled, UIDynamic target)
		{
			if (target == null) return;
			
			LayoutElement tmpLE = target.GetComponent<LayoutElement>();

			if (tmpLE != null)
			{
				if (enabled == true)
				{

					tmpLE.transform.localScale = new Vector3(1, 1, 1);
					if( !ignoreLayoutUI.Contains(target) ) tmpLE.ignoreLayout = false;
				}
				else
				{
					tmpLE.transform.localScale = new Vector3(0, 0, 0);
					tmpLE.ignoreLayout = true;
				}
			}
		}
		
		// ===========================================================================================
		// TriggerHandler implementation for easier handling of custom triggers.
		// Essentially call this in your plugin init code:
		//     StartCoroutine(SimpleTriggerHandler.LoadAssets());
		//
		// Credit to AcidBubbles for figuring out how to do custom triggers.
		//
		// hazmhox note : Edited a bit to allow the use of transitions ( original code coming from LogicBricks 4 )
		public class SimpleTriggerHandler : TriggerHandler
		{
			public static bool Loaded { get; private set; }

			private static SimpleTriggerHandler myInstance;

			private RectTransform myTriggerActionsPrefab;
			private RectTransform myTriggerActionMiniPrefab;
			private RectTransform myTriggerActionDiscretePrefab;
			private RectTransform myTriggerActionTransitionPrefab;

			public static SimpleTriggerHandler Instance {
				get {
					if (myInstance == null)
						myInstance = new SimpleTriggerHandler();
					return myInstance;
				}
			}

			public static void LoadAssets()
			{
				SuperController.singleton.StartCoroutine(Instance.LoadAssetsInternal());
			}

			private IEnumerator LoadAssetsInternal()
			{
				foreach (var x in LoadAsset("z_ui2", "TriggerActionsPanel", p => myTriggerActionsPrefab = p))
					yield return x;
				foreach (var x in LoadAsset("z_ui2", "TriggerActionMiniPanel", p => myTriggerActionMiniPrefab = p))
					yield return x;
				foreach (var x in LoadAsset("z_ui2", "TriggerActionDiscretePanel", p => myTriggerActionDiscretePrefab = p))
					yield return x;
				foreach (var x in LoadAsset("z_ui2", "TriggerActionTransitionPanel", p => myTriggerActionTransitionPrefab = p))
					yield return x;

				Loaded = true;
			}

			private IEnumerable LoadAsset(string assetBundleName, string assetName, Action<RectTransform> assign)
			{
				AssetBundleLoadAssetOperation request = AssetBundleManager.LoadAssetAsync(assetBundleName, assetName, typeof(GameObject));
				if (request == null)
					throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null request.");
				yield return request;
				GameObject go = request.GetAsset<GameObject>();
				if (go == null)
					throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null GameObject.");
				RectTransform prefab = go.GetComponent<RectTransform>();
				if (prefab == null)
					throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null RectTansform.");
				assign(prefab);
			}


			void TriggerHandler.RemoveTrigger(Trigger t)
			{
				// nothing to do
			}

			void TriggerHandler.DuplicateTrigger(Trigger t)
			{
				throw new NotImplementedException();
			}

			RectTransform TriggerHandler.CreateTriggerActionsUI()
			{
				return UnityEngine.Object.Instantiate(myTriggerActionsPrefab);
			}

			RectTransform TriggerHandler.CreateTriggerActionMiniUI()
			{
				return UnityEngine.Object.Instantiate(myTriggerActionMiniPrefab);
			}

			RectTransform TriggerHandler.CreateTriggerActionDiscreteUI()
			{
				return UnityEngine.Object.Instantiate(myTriggerActionDiscretePrefab);
			}

			RectTransform TriggerHandler.CreateTriggerActionTransitionUI()
			{
				return UnityEngine.Object.Instantiate(myTriggerActionTransitionPrefab);
			}

			void TriggerHandler.RemoveTriggerActionUI(RectTransform rt)
			{
				UnityEngine.Object.Destroy(rt?.gameObject);
			}
		}

		// Wrapper for easier handling of custom triggers.
		public class EventTrigger : Trigger
		{
			public string Name {
				get { return name; }
				set { name = value; myNeedInit = true; }
			}

			public MVRScript Owner {
				get; private set;
			}

			private string name;
			private bool myNeedInit = true;

			public EventTrigger(MVRScript owner, string name)
			{
				Name = name;
				Owner = owner;
				handler = SimpleTriggerHandler.Instance;
			}

			public void OpenPanelActionStart()
			{
				if (!SimpleTriggerHandler.Loaded)
				{
					SuperController.LogError("EventTrigger: You need to call SimpleTriggerHandler.LoadAssets() before use.");
					return;
				}

				triggerActionsParent = Owner.UITransform;
				InitTriggerUI();
				OpenTriggerActionsPanel();
				if (myNeedInit)
				{
					triggerActionsPanel.Find("Panel/Header Text").GetComponent<Text>().text = Name;
					triggerActionsPanel.Find("Content/Tab1/Label").GetComponent<Text>().text = "Event Actions";
					triggerActionsPanel.Find("Content/Tab2").gameObject.SetActive(false);
					triggerActionsPanel.Find("Content/Tab3").gameObject.SetActive(false);
					myNeedInit = false;
				}
			}
			
			public void OpenPanelTransition()
			{
				if (!SimpleTriggerHandler.Loaded)
				{
					SuperController.LogError("EventTrigger: You need to call SimpleTriggerHandler.LoadAssets() before use.");
					return;
				}

				triggerActionsParent = Owner.UITransform;
				InitTriggerUI();
				OpenTriggerActionsPanel();
				
				//getRoot(triggerActionsParent,0);
				if (myNeedInit)
				{
					triggerActionsPanel.Find("Panel/Header Text").GetComponent<Text>().text = Name;
					triggerActionsPanel.Find("Content/Tab2/Label").GetComponent<Text>().text = "Frequencies Actions";
					triggerActionsPanel.Find("Content/Tab1").gameObject.SetActive(false);
					triggerActionsPanel.Find("Content/Tab2").gameObject.SetActive(true);
					triggerActionsPanel.Find("Content/Tab2").GetComponent<Toggle>().isOn = true;					
					triggerActionsPanel.Find("Content/Tab3").gameObject.SetActive(false);
					myNeedInit = false;
				}
			}
	
			public void RestoreFromJSON(JSONClass jc, string subScenePrefix, bool isMerge, bool setMissingToDefault)
			{
				if (jc.HasKey(Name))
				{
					JSONClass tc = jc[Name].AsObject;
					if (tc != null)
						base.RestoreFromJSON(tc, subScenePrefix, isMerge);
				}
				else if (setMissingToDefault)
				{
					base.RestoreFromJSON(new JSONClass());
				}
			}

			public void Trigger()
			{
				active = true;
				active = false;
			}
			
			public void TriggerTransition(float val)
			{
				this.transitionInterpValue = val;
			}
		}
    }

}